RULE_FILE = "RESC-RULE.toml"
RULE_FILE_URL = "https://raw.githubusercontent.com/zricethezav/gitleaks/master/config/gitleaks.toml"
NAMESPACE = "resc"
RESC_HELM_REPO_URL = "https://abnamro.github.io/repository-scanner"
HELM_REPO_NAME = "resc-helm-repo"
RELEASE_NAME = "resc"
CHART_NAME = "resc"
VALUES_FILE = "custom-values.yaml"
